'**************************************************
' U-Board USB1.0/1.1 Interface board Version 1.00
' Filename: UBOARD.BAS
' Compiler: Melabs PICBASIC Pro 2.40 or higher
'
' Included file:
'	USB_CH9.ASM	USB Chapter 9 code
'	HIDCLASS.ASM	USB HID implement
'	USB_DEFS.INC	USB Defined data and ENDPOINT implement
'	USBDECS.ASM	Relocate descriptor file
'	UBOARD_D.ASM	U-Board descriptor
'**************************************************

wsave		var	byte $70 system      ' Reserved first 70H for system service
ssave		var	byte bank0 system    ' Reserved STATUS
psave		var	byte bank0 system    ' Reserved PCLATH
fsave		var	byte bank0 system    ' Reserved FSR

bufferEP1IN	    var	byte[8] ' EP1 In buffer
bufferEP1OUT	var	byte[8] ' EP1 Out buffer
bufferEP2IN	    var	byte[8] ' EP2 In buffer
bufferEP2OUT	var	byte[8] ' EP2 Out buffer
cntEP1IN    	var	byte ' EP1 In byte count
cntEP2IN        var	byte ' EP2 In byte count

ReportID_IN     Var Byte
ReportID_OUT    Var Byte

ReportData_IN   Var Byte
ReportData_OUT  Var Byte

ADC_CH          var	byte ' A/D Channel select
ADC_VALUE       var	byte ' A/D Data

'U-Board Pin assigned
LED_READY	var	PORTA.4 ' LED Ready pin (active low)
DATA_LE		var	PORTC.6 ' PORTOUT data latch enable (active high)
DATA_OE		var	PORTC.7 ' PORTIN data output enable (active low)
SHF_CLK		var	PORTC.0 ' SHIFTOUT clock (active high)
SHF_LE		var	PORTC.1 ' SHIFTOUT latch enable (active high)
SHF_DATA	var	PORTC.2 ' SHIFTOUT data pin
SHF_RESET	var	PORTA.4 ' SHIFTOUT reset parasitic LED_READY signal
                        ' but hardware invert output (active high)

DEFINE	OSC		24	' Set oscillator clock (6 MHz in H4 PLL = 24 MHz)
DEFINE	ADC_BITS	8	' Set number of bits in result (8, 10 or 12)
DEFINE	ADC_CLOCK	2	' Set clock source (2 = F INT/32)
DEFINE	ADC_SAMPLEUS	50	' Set sampling time in microseconds

Initialize:
	HIGH	SHF_RESET	' Reset SHIFTOUT Bus and LED Ready
	PORTC = %10000000	' Clear all control signals
				' DATA_LE low , DATA_OE high , all SHF_ low
	TRISC = 0		' PORTC to all outputs
	PULSOUT SHF_LE,1	' Reset SHIFTOUT Bus
	PORTB = 0		' All LEDs off
	TRISB = 0		' PORTB to all outputs
	PULSOUT DATA_LE,1	' Clear PORTOUT to logic 0
	GOTO Start		' Skip around interrupt handler

' Define interrupt handler
DEFINE	INTHAND USBINT

' Assembly language interrupt handler
ASM

; W, STATUS and PCLATH registers already saved, save FSR
USBINT	movf	FSR, W
	movwf	fsave

; Check interrupt source and vector there
	movlw	high ServiceUSBInt
	movwf	PCLATH
	btfsc	PIR1, USBIF
	call	ServiceUSBInt

; Restore saved registers
	clrf	STATUS
        movf    fsave, W
        movwf   FSR
        movf    psave, W
        movwf   PCLATH
        swapf   ssave, W
        movwf   STATUS
        swapf   wsave, F
        swapf   wsave, W
        retfie                  ; Return from interrupt
ENDASM


Start:
	ADCON1 = %00000010         '  Set PORTA0-3 and 5 to A/D input
	ADCON0 = ADCON0|%00000101 ' Set ADCON0 register
	USBInit                 ' Init USB and wait till configured
	LOW LED_READY		' Turn on LED for USB ready
	ADC_CH = 0             ' Initial A/D channel to 0
	bufferEP1IN[0] = 0     ' Set EP1
	bufferEP1OUT[0] = 1    ' Set EP1 report ID.1 (PORTIN)

Main:
	GOSUB	EP1InCheck   ' Check EP1IN
	GOSUB	EP1OutCheck  ' Check EP1OUT
	GOSUB	EP2InCheck   ' Check EP2IN
	GOSUB	EP2OutCheck  ' Check EP2OUT
	GOTO	Main         ' Loop to main

EP_NO_DATA:
	RETURN             ' Return to main

EP1InCheck:
	USBIn 1, bufferEP1IN, cntEP1IN, EP_NO_DATA ' Get EP1IN
	IF cntEP1IN <> 2 THEN  EP_NO_DATA	' Ignore packet if size <> 2

	ReportID_IN=bufferEP1IN[0]

	IF ReportID_IN >3 THEN EP_NO_DATA	' Ignore packet if Report ID. > 2
	IF ReportID_IN =0 THEN WriteReport0  ' Check byte if Report ID.0
	IF ReportID_IN =3 THEN WriteReport0  ' Check byte if Report ID.0
    bufferEP1OUT[0] = ReportID_IN ' Put ID.1 or 2 in 1st byte output
	IF ReportID_IN =2 THEN WriteReport2  ' A/D request if Report ID.2

WriteReport1:
	' Write Report ID.1,  PORTOUT & Shift data to SHIFTBUS
	TRISB = 0				' PORTB to all outputs
	PORTB = bufferEP1IN[1]			' Send data to PORTOUT
	SHIFTOUT SHF_DATA,SHF_CLK,1,[bufferEP1IN[1]\8] ' Also shift data to SHIFTBUS
	PULSOUT DATA_LE,1			' Latch PORTOUT data
	RETURN					' Return to main program

	' Write Report ID.0, O = Latch SHIFTBUS & Relocate ID.
    '                    Other = Relocate ID.
WriteReport0:
    ReportData_IN=bufferEP1IN[1]
	IF (ReportData_IN=0) or (ReportData_IN=3) THEN
      bufferEP1OUT[0] = 1 ' Report PORTOUT with ID.1
	  PULSOUT SHF_LE,1			' Latch SHIFTBUS data
    else
      bufferEP1OUT[0] = ReportData_IN ' Redirect report with specify ID.
    Endif
	RETURN					' Return to main program

	' Report ID.2, Set A/D Ch.
WriteReport2:
	ADC_CH = bufferEP1IN[1]			' Get A/D Ch. from buffer
	If ADC_CH > 4 then EP_NO_DATA		' Ignore if invalid Ch.
	RETURN					' Return to main program


EP1OutCheck:
	IF bufferEP1OUT[0]=2 THEN ReadReport2  ' A/D request if Report ID.2
    ' Read PORTIN if other ID.
ReadPORTIN:
	TRISB = %11111111			' PORTB to all inputs
	LOW DATA_OE				' PORTIN passed to PORTB
	bufferEP1OUT[1] = PORTB			' Get PORTIN data to buffer
	HIGH DATA_OE				' Inhibit PORTIN data
	TRISB = 0				' PORTB to all outputs
    GOTO Send_EP1           ' Send EP1
ReadReport2:
	ADCIN ADC_CH,ADC_VALUE     ' Convert A/D
    bufferEP1OUT[1] = ADC_VALUE ' Put A/D data to buffer
Send_EP1:
	USBOut 1, bufferEP1OUT, 2, EP_NO_DATA	' Send report back
	RETURN					' Return to main program

EP2InCheck:
	USBIn 2, bufferEP2IN, cntEP2IN, EP_NO_DATA ' Get EP2IN
	If cntEP2IN <> 2 then EP_NO_DATA   ' Ignore if data count is not 2
	if bufferEP2IN[0]<>2 then EP_NO_DATA ' Ignore data if not ID.2
	ADC_CH = bufferEP2IN[1]    ' Get A/D Ch. from buffer
	RETURN					' Return to main program

EP2OutCheck:
	If ADC_CH > 4 then EP_NO_DATA  ' Ignore if invalid Ch.
	ADCIN ADC_CH,ADC_VALUE     ' Convert A/D
	bufferEP2OUT[0] = 2        ' Report with ID.2
	bufferEP2OUT[1] = ADC_VALUE 'Put A/D data to buffer
Send_EP2:
	USBOut 2, bufferEP2OUT, 2, EP_NO_DATA	' Send the bytes back
	RETURN					' Return to main program
